#include <pic.h>
#include "LCD_Defs.h"
#include "Main.h"
#include "MCP9800 IIC.h"


static volatile unsigned char Runt;

union
{ 	
	unsigned int Full_Temp ;
	unsigned char Temp[2] ;
}MCP9800;

//*******************************************************************************************
// Read_Temperature() 
// Ū MCP9800 ū׷Pū׭
//  I2C BusAū׸ѪR 12-bit (0.0625 )
// I2C ŪRO : Start + Address + point + Re-Start + Address + Read MSB + Read Lsb + Stop
//*******************************************************************************************
void Read_Temperature(void)
{
	IdleI2C();             			// ensure module is idle
  	StartI2C();						// Start condition
	I2C_Done();						// Wait Start condition completed

	Runt=WriteI2C(MCP9800_Addr_Write);	// gJ 0x9A RO
	I2C_Done();						// Clear SSPIF flag

	Runt=WriteI2C(MCP9800_Point_Temp);	// gJŪūתRO          
	I2C_Done();						// Clear SSPIF flag

   	RestartI2C();              		// initiate Restart condition
	I2C_Done();

	Runt=WriteI2C(MCP9800_Addr_Read);	  
	I2C_Done();						// Clear SSPIF flag

	MCP9800.Temp[1]=ReadI2C();		// Enable I2C Receiver & wait BF=1 until received data
	I2C_Done();						// Clear SSPIF flag

	AckI2C();						// Genarate Non_Acknowledge to MCP9800
	I2C_Done();	   
 
	MCP9800.Temp[0]=ReadI2C();		// Enable I2C Receiver & wait BF=1 until received data
	I2C_Done();						// Clear SSPIF flag

	NotAckI2C();					// Genarate Non_Acknowledge to MCP9800
	I2C_Done();	  
                     
    StopI2C();                  	// send STOP condition
	I2C_Done();						// wait until stop condition is over 
	
	 
}

//*************************************************************************
// Config the MCP9800 as 12-bit resolution with 0.0625C
// Write command: Start + Address + Point + Condig Data + Stop
//*************************************************************************
void Write_MCP9800_Config(void)
{
	IdleI2C();             			// ensure module is idle
  	StartI2C();						// Start condition
	I2C_Done();						// Wait Start condition completed

	Runt=WriteI2C(MCP9800_Addr_Write);	  
	I2C_Done();					

	Runt=WriteI2C(MCP9800_Point_Conf);	          
	I2C_Done();						

   	Runt=WriteI2C(0b01100000);		// 12-bit resolotion without Shutdown.  
	I2C_Done();						// Clear SSPIF flag

	NotAckI2C();					// Genarate Non_Acknowledge to MCP9800
	I2C_Done();	  
                     
    StopI2C();                  	// send STOP condition
	I2C_Done();						// wait until stop condition is over 	 
}

//********************************************************************
// As below are I2C function
// These functions were modified from C18 I2C Libyaries' Source Code
//********************************************************************
//**** Test and wait until I2C module is idle.
void IdleI2C( void ) 				
{
  while ( ( SSPCON2 & 0x1F ) || ( R_nW ) )
     continue;
}

//Send I2C bus start condition.
void StartI2C( void )
{
  SEN = 1;            // initiate bus start condition
}

//This routine writes a single byte to the I2C bus.
unsigned char WriteI2C( unsigned char data_out )	
{
  SSPBUF = data_out;           	// write single byte to SSPBUF
  if (WCOL)      				// test if write collision occurred
   return ( -1 );              	// if WCOL bit is set return negative #
  else
  {
    while(BF);   				// wait until write cycle is complete         
    IdleI2C();                 	// ensure module is idle
    if ( SSPCON2bits.ACKSTAT ) 	// test for ACK condition received
    	 return ( -2 );
	else return ( 0 );          // if WCOL bit is not set return non-negative #
  }
}

unsigned char ReadI2C( void )
{
  RCEN = 1;           // enable master for 1 byte reception
  while ( !BF );      // wait until byte received  
  return ( SSPBUF );              // return with read byte 
}

void AckI2C( void )
{
  ACKDT = 0;           // set acknowledge bit state for ACK
  ACKEN = 1;           // initiate bus acknowledge sequence
}

void NotAckI2C( void )
{
  ACKDT = 1;          // set acknowledge bit for not ACK
  ACKEN = 1;          // initiate bus acknowledge sequence
}

void RestartI2C( void )
{
  RSEN = 1;           // initiate bus restart condition
}
void StopI2C( void )
{
  PEN = 1;            // initiate bus stop condition
}

void I2C_Done(void)
{
	while (!SSPIF);			// Completed the action when the SSPIF is Hi.
	SSPIF=0;   				// Clear SSPIF
}

//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @4MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
 		SCL=1;				// Initial PortC,bit 3 & 4 as Input with Open Drain
		SDA=1;				// RC3 = SCL  ,  RC4 = SDA
		SCL_DIR =1;			 
		SDA_DIR =1;
 
		SSPCON=0b00101000;	// Enable SSP, I2C Master Mode, Clock Rate: FOSC/(4*SSPADD+1)
 		SSPSTAT=0b10000000; // Disable slew rate for 100KHz,and clear status bit
     	SSPCON2=0b00000000;	// Set SCL,SDA into Ready status
 	   	SSPCON3=0b00000000;  
		SSPADD= 79;			// (32MHz/4) / (79+1)= 100KHz  
		SSPMSK= 0xFF;
}

