#include <xc.h>
#include "LCD_Defs.h"
#include "Main.h"
#include "MCP9800 IIC.h"

__CONFIG(FOSC_INTOSC & WDTE_OFF & PWRTE_OFF & MCLRE_ON & CP_OFF & BOREN_ON & CLKOUTEN_ON & IESO_OFF & FCMEN_OFF);
__CONFIG(FCMEN_OFF & VCAPEN_OFF & PLLEN_ON & STVREN_OFF & BORV_LO & LVP_OFF);

extern union
{
    unsigned int Word;
    unsigned char Byte[2];
} ADC_Result;

extern union
{
    unsigned int Full_Temp;
    unsigned char Temp[2];
} MCP9800;

extern union
{
    unsigned int Full_Timer;
    unsigned char Timer_Byte[2];
} CSM;

extern unsigned char CSM_Flag;
extern unsigned int CSM_BUF[16];
extern unsigned char Key_Input;
extern unsigned char CSM_Key;
extern unsigned int CSM_ever_Freq_Temp;
extern unsigned char Disp_Count;

unsigned long Temp_Buff;
unsigned int Disp_Temp;
unsigned char Dig1;
unsigned int CSM_ever_Freq;
unsigned char Disp_Count = 0;
unsigned int *CSM_Ptr;
unsigned char i;

void main(void)
{
    OSCCONbits.IRCF = 0B1110; // 8MHz * 4PLL = 32MHz(Fosc)

    Initialize_CPU();

    Cap_Init();
    Init_Timer4();

    Initialize_I2C_Master(); // 設定 Master IIC 讀寫 MCP9800 
    Write_MCP9800_Config(); // 設定 MCP9800 的工作模式
    Initialize_LCD();

    PEIE = 1;
    GIE = 1;
    Key_Input = 0x01; // 先執行 ADC 轉換電壓顯示   

    while (1)
    {
        if (Key_Input == 0x01) // S5 按下時，LED6 亮
        {
            LED6_On;
            ;
            NOP();
            LED7_Off;
            Key_Input = 0; // 清除按鍵狀態為無按鍵

            Show_MCP9800_Temperature();
        }

        if (Key_Input == 0x02) // S6 按下時，LED7 亮
        {
            LED6_Off;
            NOP();
            LED7_On;
            Key_Input = 0;

            Show_CSM_Frequency();

        }

        if (CSM_Key == 1) LED8_On; // 觸控按鍵按下, 且平均頻率低於 350KHz 點亮 LED8
        else LED8_Off;

    }
}

void Show_MCP9800_Temperature(void)
{
    Disp_Count = 0; // 第一次不延遲，立即顯示溫度值
    Clear_LCD();

    do
    {
        if (Disp_Count == 0) // 600mS 到了嗎? 本程式會每 600mS 更新 LCD 上的顯示頻率溫度一次
        {
            Read_Temperature(); // 讀取 MCP9800 溫度
            Temp_Buff = MCP9800.Full_Temp >> 4; // 蓋掉最右邊 4 個無用的位元
            Disp_Temp = (((unsigned long) Temp_Buff)*625) / 100; // 12-bit 解析度，1 LSB=0.0625 度
            // 算出實際溫度並取小數兩位
            Dig1 = 0;

            Dig1 = Disp_Temp / 10000; // 轉換百位溫度
            if (Dig1 == 0) BlankDigit1(); // 如百位溫度為零，不顯示
            else DisplayDigit1(Dig1); // 顯示百位溫度在 LCD 第一數字上
            Disp_Temp = Disp_Temp % 10000; // 取出十位以後的數

            Dig1 = Disp_Temp / 1000;
            if (Dig1 == 0) BlankDigit2(); // 顯示十位數
            else DisplayDigit2(Dig1);
            Disp_Temp = Disp_Temp % 1000; // 取出十位以後的數


            DisplayDigit3((Disp_Temp / 100)); // 顯示個位數
            Disp_Temp = Disp_Temp % 100;

            DisplayDigit4(Disp_Temp / 10); // 顯示十分數
            Disp_Temp = Disp_Temp % 10;

            // DisplayDigit5(Disp_Temp);   // 顯示百分數

            SEG_3DP = 1; // 點小數點
            SEG_5A = 0;
            SEG_5B = 0;
            SEG_5C = 0;
            SEG_5D = 1;
            SEG_5E = 1;
            SEG_5F = 0;
            SEG_5G = 1; // 顯示度的符號

            SEG_oC = !SEG_oC;
            Disp_Count = 120; // 設定每 600mS 顯示溫度一次 
        }
    }
    while (Key_Input == 0); // 迴圈內檢查是否有新按鍵按下? 若無則繼續顯示溫度
    //若有則跳出顯示溫度函數
}

void Show_CSM_Frequency(void)
{
    Disp_Count = 0;
    Clear_LCD();

    do
    {
        if (CSM_Key == 1) LED8_On; // 觸控按鍵按下, 且平均頻率低於 350KHz 點亮 LED8
        else LED8_Off;

        if ((Disp_Count == 0) && CSM_Flag)
        {
            CSM_Flag = 0;
            CSM_Ptr = &CSM_BUF[0];
            CSM_ever_Freq = 0;


            for (i = 0; i <= 15; i++)
            {
                CSM_ever_Freq = *CSM_Ptr++ + CSM_ever_Freq; //CSM 頻率取 16 次的平均值
            }

            if (!Disp_Count) // 100mS 計時到了嗎?
            {
                Disp_Count = 20; // Set 100mS 更新 LCD 顯示頻率一次

                CSM_ever_Freq = CSM_ever_Freq >> 4; // 加總後的頻率值除以 16，取平均
                CSM_ever_Freq_Temp = CSM_ever_Freq;

                SEG_1A = 1;
                SEG_1B = 0;
                SEG_1C = 0;
                SEG_1D = 0;
                SEG_1E = 1;
                SEG_1F = 1;
                SEG_1G = 1; //Display "F"
                SEG_2A = 0;
                SEG_2B = 0;
                SEG_2C = 0;
                SEG_2D = !SEG_2D;
                SEG_2E = 0;
                SEG_2F = 0;
                SEG_2G = !SEG_2G; //Display "="

                DisplayDigit3((CSM_ever_Freq / 100)); // 顯示個位數
                CSM_ever_Freq = CSM_ever_Freq % 100;

                DisplayDigit4(CSM_ever_Freq / 10); // 顯示十分數
                CSM_ever_Freq = CSM_ever_Freq % 10;

                DisplayDigit5(CSM_ever_Freq); // 顯示個位數

            }

            Disp_Count = 40; // 設定每 200mS 顯示電壓一次 
        }
    }
    while (Key_Input == 0); // 迴圈內檢查是否有新按鍵按下? 若無則繼續顯示電壓         //若有則跳出顯示電壓函數
}

void Delay_Second(int Delay_Count)
{
    unsigned char i;

    for (i = 0; i <= Delay_Count; i++)
    {
        Delay_Long();
    }
}

void Delay_Long(void)
{
    int Delay1, Delay2;

    for (Delay1 = 0; Delay1 < 20000; Delay1++);
}

void Initialize_CPU(void)
{
    ANSELA = 0x00; // 暫時關閉 ADC 所有輸入腳 
    ANSELB = 0x00;
    ANSELD = 0x00;
    ADCON1 = 0x50;

    TRISB4 = 0; // output for COM0
    NOP();
    TRISB5 = 0; // output for COM1
    NOP();
    TRISA2 = 0; // output for COM2
    NOP();
    TRISD0 = 0; // output for COM3
    NOP();

    TRISD1 = 1; // Input for SW5
    NOP();
    TRISD2 = 1; // Inout for SW6
    NOP();
    TRISC6 = 0; // Output for LED6
    NOP();
    TRISC7 = 0; // Output for LED7
    NOP();
    TRISB3 = 0; // Output for LED8
    NOP();

    LED6_On;
    NOP();
    LED7_On;
    NOP();
    LED8_On;
    NOP();
    Delay_Second(20);
    LED6_Off;
    NOP();
    LED7_Off;
    NOP();
    LED8_Off;
    NOP();
    // Configure PR2 to 31.2Khz PWM 


}

